@@ -1,23 +1,20 @@
-/* Copyright (C) 2000, Ghostgum Software Pty Ltd.  All rights reserved.
+/* Copyright (C) 2000, 2001, Ghostgum Software Pty Ltd.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: dscparse.c,v $ $Revision: 1.1.2.3 $ */
+/*$Id: dscparse.c,v 1.7.2.1 2002/02/22 19:45:55 ray Exp $ */
 
 /*
  * This is a DSC parser, based on the DSC 3.0 spec, 
@@ -81,47 +78,47 @@
 #define CDSC_NEEDMORE 11
 
 /* local prototypes */
-private void * dsc_memalloc(P2(CDSC *dsc, size_t size));
-private void dsc_memfree(P2(CDSC*dsc, void *ptr));
-private CDSC * dsc_init2(P1(CDSC *dsc));
-private void dsc_reset(P1(CDSC *dsc));
-private void dsc_section_join(P3(unsigned long begin, unsigned long *pend, unsigned long **pplast));
-private int dsc_read_line(P1(CDSC *dsc));
-private int dsc_read_doseps(P1(CDSC *dsc));
-private char * dsc_alloc_string(P3(CDSC *dsc, const char *str, int len));
-private char * dsc_add_line(P3(CDSC *dsc, const char *line, unsigned int len));
-private char * dsc_copy_string(P5(char *str, unsigned int slen, 
+dsc_private void * dsc_memalloc(P2(CDSC *dsc, size_t size));
+dsc_private void dsc_memfree(P2(CDSC*dsc, void *ptr));
+dsc_private CDSC * dsc_init2(P1(CDSC *dsc));
+dsc_private void dsc_reset(P1(CDSC *dsc));
+dsc_private void dsc_section_join(P3(unsigned long begin, unsigned long *pend, unsigned long **pplast));
+dsc_private int dsc_read_line(P1(CDSC *dsc));
+dsc_private int dsc_read_doseps(P1(CDSC *dsc));
+dsc_private char * dsc_alloc_string(P3(CDSC *dsc, const char *str, int len));
+dsc_private char * dsc_add_line(P3(CDSC *dsc, const char *line, unsigned int len));
+dsc_private char * dsc_copy_string(P5(char *str, unsigned int slen, 
     char *line, unsigned int len, unsigned int *offset));
-private GSDWORD dsc_get_dword(P1(const unsigned char *buf));
-private GSWORD dsc_get_word(P1(const unsigned char *buf));
-private int dsc_get_int(P3(const char *line, unsigned int len, unsigned int *offset));
-private float dsc_get_real(P3(const char *line, unsigned int len, 
+dsc_private GSDWORD dsc_get_dword(P1(const unsigned char *buf));
+dsc_private GSWORD dsc_get_word(P1(const unsigned char *buf));
+dsc_private int dsc_get_int(P3(const char *line, unsigned int len, unsigned int *offset));
+dsc_private float dsc_get_real(P3(const char *line, unsigned int len, 
     unsigned int *offset));
-private int dsc_stricmp(P2(const char *s, const char *t));
-private void dsc_unknown(P1(CDSC *dsc)); 
-private int dsc_parse_pages(P1(CDSC *dsc));
-private int dsc_parse_bounding_box(P3(CDSC *dsc, CDSCBBOX** pbbox, int offset));
-private int dsc_parse_orientation(P3(CDSC *dsc, unsigned int *porientation, 
+dsc_private int dsc_stricmp(P2(const char *s, const char *t));
+dsc_private void dsc_unknown(P1(CDSC *dsc)); 
+dsc_private int dsc_parse_pages(P1(CDSC *dsc));
+dsc_private int dsc_parse_bounding_box(P3(CDSC *dsc, CDSCBBOX** pbbox, int offset));
+dsc_private int dsc_parse_orientation(P3(CDSC *dsc, unsigned int *porientation, 
     int offset));
-private int dsc_parse_order(P1(CDSC *dsc));
-private int dsc_parse_media(P2(CDSC *dsc, const CDSCMEDIA **page_media));
-private int dsc_parse_document_media(P1(CDSC *dsc));
-private int dsc_parse_viewer_orientation(P2(CDSC *dsc, CDSCCTM **pctm));
-private int dsc_parse_page(P1(CDSC *dsc));
-private void dsc_save_line(P1(CDSC *dsc));
-private int dsc_scan_type(P1(CDSC *dsc));
-private int dsc_scan_comments(P1(CDSC *dsc));
-private int dsc_scan_preview(P1(CDSC *dsc));
-private int dsc_scan_defaults(P1(CDSC *dsc));
-private int dsc_scan_prolog(P1(CDSC *dsc));
-private int dsc_scan_setup(P1(CDSC *dsc));
-private int dsc_scan_page(P1(CDSC *dsc));
-private int dsc_scan_trailer(P1(CDSC *dsc));
-private int dsc_error(P4(CDSC *dsc, unsigned int explanation, 
+dsc_private int dsc_parse_order(P1(CDSC *dsc));
+dsc_private int dsc_parse_media(P2(CDSC *dsc, const CDSCMEDIA **page_media));
+dsc_private int dsc_parse_document_media(P1(CDSC *dsc));
+dsc_private int dsc_parse_viewing_orientation(P2(CDSC *dsc, CDSCCTM **pctm));
+dsc_private int dsc_parse_page(P1(CDSC *dsc));
+dsc_private void dsc_save_line(P1(CDSC *dsc));
+dsc_private int dsc_scan_type(P1(CDSC *dsc));
+dsc_private int dsc_scan_comments(P1(CDSC *dsc));
+dsc_private int dsc_scan_preview(P1(CDSC *dsc));
+dsc_private int dsc_scan_defaults(P1(CDSC *dsc));
+dsc_private int dsc_scan_prolog(P1(CDSC *dsc));
+dsc_private int dsc_scan_setup(P1(CDSC *dsc));
+dsc_private int dsc_scan_page(P1(CDSC *dsc));
+dsc_private int dsc_scan_trailer(P1(CDSC *dsc));
+dsc_private int dsc_error(P4(CDSC *dsc, unsigned int explanation, 
     char *line, unsigned int line_len));
 
 /* DSC error reporting */
-private const int dsc_severity[] = {
+dsc_private const int dsc_severity[] = {
     CDSC_ERROR_WARN, 	/* CDSC_MESSAGE_BBOX */
     CDSC_ERROR_WARN, 	/* CDSC_MESSAGE_EARLY_TRAILER */
     CDSC_ERROR_WARN, 	/* CDSC_MESSAGE_EARLY_EOF */
@@ -553,6 +550,7 @@ dsc_add_page(CDSC *dsc, int ordinal, char *label)
     dsc->page[dsc->page_count].ordinal = ordinal;
     dsc->page[dsc->page_count].media = NULL;
     dsc->page[dsc->page_count].bbox = NULL;
+    dsc->page[dsc->page_count].viewing_orientation = NULL;
 
     dsc->page_count++;
     if (dsc->page_count >= dsc->page_chunk_length) {
@@ -663,7 +661,7 @@ dsc_set_page_bbox(CDSC *dsc, unsigned int page_number,
 /* Private functions below here.                                  */
 /******************************************************************/
 
-private void *
+dsc_private void *
 dsc_memalloc(CDSC *dsc, size_t size)
 {
     if (dsc->memalloc)
@@ -671,7 +669,7 @@ dsc_memalloc(CDSC *dsc, size_t size)
     return malloc(size);
 }
 
-private void
+dsc_private void
 dsc_memfree(CDSC*dsc, void *ptr)
 {
     if (dsc->memfree) 
@@ -681,7 +679,7 @@ dsc_memfree(CDSC*dsc, void *ptr)
 }
 
 /* private constructor */
-private CDSC *
+dsc_private CDSC *
 dsc_init2(CDSC *dsc)
 {
     dsc_reset(dsc);
@@ -717,7 +715,7 @@ dsc_init2(CDSC *dsc)
 }
 
 
-private void 
+dsc_private void 
 dsc_reset(CDSC *dsc)
 {
     unsigned int i;
@@ -752,8 +750,8 @@ dsc_reset(CDSC *dsc)
 
 	if (dsc->page[i].bbox)
 	    dsc_memfree(dsc, dsc->page[i].bbox);
-	if (dsc->page[i].viewer_orientation)
-	    dsc_memfree(dsc, dsc->page[i].viewer_orientation);
+	if (dsc->page[i].viewing_orientation)
+	    dsc_memfree(dsc, dsc->page[i].viewing_orientation);
     }
     if (dsc->page)
 	dsc_memfree(dsc, dsc->page);
@@ -764,9 +762,9 @@ dsc_reset(CDSC *dsc)
     dsc->page_pages = 0;
     dsc->page_order = CDSC_ORDER_UNKNOWN;
     dsc->page_orientation = CDSC_ORIENT_UNKNOWN;
-    if (dsc->viewer_orientation)
-	dsc_memfree(dsc, dsc->viewer_orientation);
-    dsc->viewer_orientation = NULL;
+    if (dsc->viewing_orientation)
+	dsc_memfree(dsc, dsc->viewing_orientation);
+    dsc->viewing_orientation = NULL;
 	
     /* page_media is pointer to an element of media or dsc_known_media */
     /* do not free it. */
@@ -849,7 +847,7 @@ dsc_reset(CDSC *dsc)
 * pend is a pointer to the end of this section
 * pplast is a pointer to a pointer of the end of the previous section
 */
-private void 
+dsc_private void 
 dsc_section_join(unsigned long begin, unsigned long *pend, unsigned long **pplast)
 {
     if (begin)
@@ -860,7 +858,7 @@ dsc_section_join(unsigned long begin, unsigned long *pend, unsigned long **pplas
 
 
 /* return value is 0 if no line available, or length of line */
-private int
+dsc_private int
 dsc_read_line(CDSC *dsc)
 {
     char *p, *last;
@@ -1028,7 +1026,7 @@ dsc_read_line(CDSC *dsc)
 
 
 /* Save last DSC line, for use with %%+ */
-private void 
+dsc_private void 
 dsc_save_line(CDSC *dsc)
 {
     int len = min(sizeof(dsc->last_line), dsc->line_length);
@@ -1036,7 +1034,7 @@ dsc_save_line(CDSC *dsc)
 }
 
 /* display unknown DSC line */
-private void 
+dsc_private void 
 dsc_unknown(CDSC *dsc)
 {
     if (dsc->debug_print_fn) {
@@ -1075,7 +1073,7 @@ dsc_is_section(char *line)
 }
 
 
-private GSDWORD
+dsc_private GSDWORD
 dsc_get_dword(const unsigned char *buf)
 {
     GSDWORD dw;
@@ -1086,7 +1084,7 @@ dsc_get_dword(const unsigned char *buf)
     return dw;
 }
 
-private GSWORD
+dsc_private GSWORD
 dsc_get_word(const unsigned char *buf)
 {
     GSWORD w;
@@ -1095,7 +1093,7 @@ dsc_get_word(const unsigned char *buf)
     return w;
 }
 
-private int
+dsc_private int
 dsc_read_doseps(CDSC *dsc)
 {
     unsigned char *line = (unsigned char *)dsc->line;
@@ -1119,12 +1117,17 @@ dsc_read_doseps(CDSC *dsc)
     /* skip from current position to start of PostScript section */
     dsc->skip_bytes = dsc->doseps->ps_begin - 30;
 
+    if (dsc->doseps->tiff_begin)
+	dsc->preview = CDSC_TIFF;
+    if (dsc->doseps->wmf_begin)
+	dsc->preview = CDSC_WMF;
+
     return CDSC_OK;
 }
 
 
 
-private int 
+dsc_private int 
 dsc_parse_pages(CDSC *dsc)
 {
     int ip, io; 
@@ -1215,7 +1218,7 @@ dsc_parse_pages(CDSC *dsc)
     return CDSC_OK;
 }
 
-private int 
+dsc_private int 
 dsc_parse_bounding_box(CDSC *dsc, CDSCBBOX** pbbox, int offset)
 {
     unsigned int i, n;
@@ -1346,7 +1349,7 @@ dsc_parse_bounding_box(CDSC *dsc, CDSCBBOX** pbbox, int offset)
     return CDSC_OK;
 }
 
-private int 
+dsc_private int 
 dsc_parse_orientation(CDSC *dsc, unsigned int *porientation, int offset)
 {
     char *p;
@@ -1407,7 +1410,7 @@ dsc_parse_orientation(CDSC *dsc, unsigned int *porientation, int offset)
     return CDSC_OK;
 }
 
-private int 
+dsc_private int 
 dsc_parse_order(CDSC *dsc)
 {
     char *p;
@@ -1474,7 +1477,7 @@ dsc_parse_order(CDSC *dsc)
 }
 
 
-private int 
+dsc_private int 
 dsc_parse_media(CDSC *dsc, const CDSCMEDIA **page_media)
 {
     char media_name[MAXSTR];
@@ -1497,7 +1500,7 @@ dsc_parse_media(CDSC *dsc, const CDSCMEDIA **page_media)
 }
 
 
-private int 
+dsc_private int 
 dsc_parse_document_media(CDSC *dsc)
 {
     unsigned int i, n;
@@ -1557,11 +1560,11 @@ dsc_parse_document_media(CDSC *dsc)
     return CDSC_OK;
 }
 
-/* viewer orientation is believed to be the first four elements of
+/* viewing orientation is believed to be the first four elements of
  * a CTM matrix
  */
-private int 
-dsc_parse_viewer_orientation(CDSC *dsc, CDSCCTM **pctm)
+dsc_private int 
+dsc_parse_viewing_orientation(CDSC *dsc, CDSCCTM **pctm)
 {
     CDSCCTM ctm;
     unsigned int i, n;
@@ -1571,7 +1574,7 @@ dsc_parse_viewer_orientation(CDSC *dsc, CDSCCTM **pctm)
 	*pctm = NULL;
     }
 
-    n = IS_DSC(dsc->line, "%%+") ? 3 : 20;  /* %%ViewerOrientation: */
+    n = IS_DSC(dsc->line, "%%+") ? 3 : 21;  /* %%ViewingOrientation: */
     while (IS_WHITE(dsc->line[n]))
 	n++;
 
@@ -1603,7 +1606,7 @@ dsc_parse_viewer_orientation(CDSC *dsc, CDSCCTM **pctm)
  * need to skip a binary header which contains a new line
  * character
  */
-private int 
+dsc_private int 
 dsc_scan_type(CDSC *dsc)
 {
     unsigned char *p;
@@ -1715,13 +1718,13 @@ dsc_scan_type(CDSC *dsc)
     if (dsc_read_line(dsc) <= 0)
 	return CDSC_NEEDMORE;
 	
-    dsc->dsc_version = dsc_add_line(dsc, (const char *)line, dsc->line_length);
-    if (COMPARE(line, "%!PS-Adobe")) {
+    dsc->dsc_version = dsc_add_line(dsc, dsc->line, dsc->line_length);
+    if (COMPARE(dsc->line, "%!PS-Adobe")) {
 	dsc->dsc = TRUE;
 	dsc->begincomments = DSC_START(dsc);
 	if (dsc->dsc_version == NULL)
 	    return CDSC_ERROR;	/* no memory */
-	p = line + 14;
+	p = (unsigned char *)dsc->line + 14;
 	while (IS_WHITE(*p))
 	    p++;
 	if (COMPARE(p, "EPSF-"))
@@ -1729,7 +1732,7 @@ dsc_scan_type(CDSC *dsc)
 	dsc->scan_section = scan_comments;
 	return CDSC_PSADOBE;
     }
-    if (COMPARE(line, "%!")) {
+    if (COMPARE(dsc->line, "%!")) {
 	dsc->scan_section = scan_comments;
 	return CDSC_NOTDSC;
     }
@@ -1740,7 +1743,7 @@ dsc_scan_type(CDSC *dsc)
 
 
 
-private int 
+dsc_private int 
 dsc_scan_comments(CDSC *dsc)
 {
     /* Comments section ends at */
@@ -2007,6 +2010,8 @@ dsc_scan_comments(CDSC *dsc)
     else if (IS_DSC(line, "%%DocumentData:")) {
 	unsigned int n = continued ? 3 : 15;
 	char *p = dsc->line + n;
+        while (IS_WHITE(*p))
+	    p++;
 	dsc->id = CDSC_DOCUMENTDATA;
 	if (COMPARE(p, "Clean7Bit"))
 	    dsc->document_data = CDSC_CLEAN7BIT;
@@ -2043,7 +2048,7 @@ dsc_scan_comments(CDSC *dsc)
 }
 
 
-private int 
+dsc_private int 
 dsc_scan_preview(CDSC *dsc)
 {
     /* Preview section ends at */
@@ -2061,6 +2066,9 @@ dsc_scan_preview(CDSC *dsc)
 	    dsc->beginpreview = DSC_START(dsc);
 	    dsc->endpreview = DSC_END(dsc);
 	    dsc->scan_section = scan_preview;
+	    /* Don't mark the preview as EPSI if a DOS EPS header is present */
+	    if (dsc->preview == CDSC_NOPREVIEW)
+		dsc->preview = CDSC_EPSI;
 	    return CDSC_OK;
 	}
 	else {
@@ -2096,7 +2104,7 @@ dsc_scan_preview(CDSC *dsc)
     return CDSC_OK;
 }
 
-private int
+dsc_private int
 dsc_scan_defaults(CDSC *dsc)
 {
     /* Defaults section ends at */
@@ -2157,9 +2165,9 @@ dsc_scan_defaults(CDSC *dsc)
 	if (dsc_parse_bounding_box(dsc, &(dsc->page_bbox), 18))
 	    return CDSC_ERROR;
     }
-    else if (IS_DSC(line, "%%ViewerOrientation:")) {
-	dsc->id = CDSC_VIEWERORIENTATION;
-	if (dsc_parse_viewer_orientation(dsc, &dsc->viewer_orientation))
+    else if (IS_DSC(line, "%%ViewingOrientation:")) {
+	dsc->id = CDSC_VIEWINGORIENTATION;
+	if (dsc_parse_viewing_orientation(dsc, &dsc->viewing_orientation))
 	    return CDSC_ERROR;
     }
     else {
@@ -2173,7 +2181,7 @@ dsc_scan_defaults(CDSC *dsc)
 
 /* CDSC_RESPONSE_OK and CDSC_RESPONSE_CANCEL mean ignore the 
  * mismatch (default) */
-private int
+dsc_private int
 dsc_check_match_prompt(CDSC *dsc, const char *str, int count)
 {
     if (count != 0) {
@@ -2188,7 +2196,7 @@ dsc_check_match_prompt(CDSC *dsc, const char *str, int count)
     return CDSC_RESPONSE_CANCEL;
 }
 
-private int
+dsc_private int
 dsc_check_match_type(CDSC *dsc, const char *str, int count)
 {
     if (dsc_check_match_prompt(dsc, str, count) == CDSC_RESPONSE_IGNORE_ALL)
@@ -2198,7 +2206,7 @@ dsc_check_match_type(CDSC *dsc, const char *str, int count)
 
 /* complain if Begin/End blocks didn't match */
 /* return non-zero if we should ignore all DSC */
-private int
+dsc_private int
 dsc_check_match(CDSC *dsc)
 {
     int rc = 0;
@@ -2224,7 +2232,7 @@ dsc_check_match(CDSC *dsc)
 }
 
 
-private int 
+dsc_private int 
 dsc_scan_prolog(CDSC *dsc)
 {
     /* Prolog section ends at */
@@ -2324,7 +2332,7 @@ dsc_scan_prolog(CDSC *dsc)
     return CDSC_OK;
 }
 
-private int
+dsc_private int
 dsc_scan_setup(CDSC *dsc)
 {
     /* Setup section ends at */
@@ -2460,7 +2468,7 @@ dsc_scan_setup(CDSC *dsc)
     return CDSC_OK;
 }
 
-private int 
+dsc_private int 
 dsc_scan_page(CDSC *dsc)
 {
     /* Page section ends at */
@@ -2672,10 +2680,10 @@ dsc_scan_page(CDSC *dsc)
 	if (dsc_parse_bounding_box(dsc, &dsc->page[dsc->page_count-1].bbox, 18))
 	    return CDSC_NOTDSC;
     }
-    else if (IS_DSC(line, "%%ViewerOrientation:")) {
-	dsc->id = CDSC_VIEWERORIENTATION;
-	if (dsc_parse_viewer_orientation(dsc, 
-	    &dsc->page[dsc->page_count-1].viewer_orientation))
+    else if (IS_DSC(line, "%%ViewingOrientation:")) {
+	dsc->id = CDSC_VIEWINGORIENTATION;
+	if (dsc_parse_viewing_orientation(dsc, 
+	    &dsc->page[dsc->page_count-1].viewing_orientation))
 	    return CDSC_ERROR;
     }
     else if (IS_DSC(line, "%%BeginFont:")) {
@@ -2774,7 +2782,7 @@ dsc_scan_page(CDSC *dsc)
  * %%PageResources: 
  */
 
-private int
+dsc_private int
 dsc_scan_trailer(CDSC *dsc)
 {
     /* Trailer section start at */
@@ -2868,28 +2876,24 @@ dsc_scan_trailer(CDSC *dsc)
 	    return CDSC_ERROR;
     }
     else if (IS_DSC(dsc->line, "%%Page:")) {
-	dsc->id = CDSC_PAGE;
 	/* This should not occur in the trailer, but we might see 
 	 * this if a document has been incorrectly embedded.
 	 */
-	dsc_unknown(dsc);
-	if (dsc->page_count) {
-	    int rc = dsc_error(dsc, CDSC_MESSAGE_PAGE_IN_TRAILER, 
-		    dsc->line, dsc->line_length);
-	    switch (rc) {
-		case CDSC_RESPONSE_OK:
-		    /* Assume that we are really in the previous */
-		    /* page, not the trailer */
-		    dsc->scan_section = scan_pre_pages;
-		    if (dsc->page_count)
-			dsc->page[dsc->page_count-1].end = DSC_START(dsc);
-		    break;
-		case CDSC_RESPONSE_CANCEL:
-		    /* ignore pages in trailer */
-		    break;
-		case CDSC_RESPONSE_IGNORE_ALL:
-		    return CDSC_NOTDSC;
-	    }
+	int rc = dsc_error(dsc, CDSC_MESSAGE_PAGE_IN_TRAILER, 
+		dsc->line, dsc->line_length);
+	switch (rc) {
+	    case CDSC_RESPONSE_OK:
+		/* Assume that we are really in the previous */
+		/* page, not the trailer */
+		dsc->scan_section = scan_pre_pages;
+		if (dsc->page_count)
+		    dsc->page[dsc->page_count-1].end = DSC_START(dsc);
+		return CDSC_PROPAGATE;	/* try again */
+	    case CDSC_RESPONSE_CANCEL:
+		/* ignore pages in trailer */
+		break;
+	    case CDSC_RESPONSE_IGNORE_ALL:
+		return CDSC_NOTDSC;
 	}
     }
     else if (IS_DSC(line, "%%DocumentNeededFonts:")) {
@@ -2911,7 +2915,7 @@ dsc_scan_trailer(CDSC *dsc)
 }
 
 
-private char *
+dsc_private char *
 dsc_alloc_string(CDSC *dsc, const char *str, int len)
 {
     char *p;
@@ -2959,7 +2963,7 @@ dsc_alloc_string(CDSC *dsc, const char *str, int len)
 }
 
 /* store line, ignoring leading spaces */
-private char *
+dsc_private char *
 dsc_add_line(CDSC *dsc, const char *line, unsigned int len)
 {
     char *newline;
@@ -2992,7 +2996,7 @@ dsc_add_line(CDSC *dsc, const char *line, unsigned int len)
 /* Return pointer to string  */
 /* Store number of used characters from line */
 /* Don't copy enclosing () */
-private char *
+dsc_private char *
 dsc_copy_string(char *str, unsigned int slen, char *line, 
 	unsigned int len, unsigned int *offset)
 {
@@ -3082,7 +3086,7 @@ dsc_copy_string(char *str, unsigned int slen, char *line,
     return str;
 }
 
-private int 
+dsc_private int 
 dsc_get_int(const char *line, unsigned int len, unsigned int *offset)
 {
     char newline[MAXSTR];
@@ -3108,7 +3112,7 @@ dsc_get_int(const char *line, unsigned int len, unsigned int *offset)
     return atoi(newline);
 }
 
-private float 
+dsc_private float 
 dsc_get_real(const char *line, unsigned int len, unsigned int *offset)
 {
     char newline[MAXSTR];
@@ -3137,7 +3141,7 @@ dsc_get_real(const char *line, unsigned int len, unsigned int *offset)
     return (float)atof(newline);
 }
 
-private int
+dsc_private int
 dsc_stricmp(const char *s, const char *t)
 {
     while (toupper(*s) == toupper(*t)) {
@@ -3150,7 +3154,7 @@ dsc_stricmp(const char *s, const char *t)
 }
 
 
-private int
+dsc_private int
 dsc_parse_page(CDSC *dsc)
 {
     char *p;
@@ -3219,7 +3223,7 @@ dsc_debug_print(CDSC *dsc, const char *str)
  *   CDSC_RESPONSE_IGNORE_ALL  Ignore all DSC.
  */
 /* Silent operation.  Don't display errors. */
-private int 
+dsc_private int 
 dsc_error(CDSC *dsc, unsigned int explanation, 
 	char *line, unsigned int line_len)
 {
