@@ -1,23 +1,20 @@
 /* Copyright (C) 1990, 1992, 1993 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdev8510.c,v $ $Revision: 1.2.2.1 $*/
+/*$Id: gdev8510.c,v 1.3.2.1 2002/02/22 19:45:55 ray Exp $*/
 /*
  * C.Itoh M8510 printer driver for ghostscript.
  *
@@ -28,7 +25,7 @@
 
 /* The device descriptor */
 private dev_proc_print_page(m8510_print_page);
-gx_device_printer far_data gs_m8510_device =
+const gx_device_printer far_data gs_m8510_device =
 	prn_device(prn_std_procs, "m8510",
 		85,				/* width_10ths, 8.5" */
 		110,				/* height_10ths, 11" */
