@@ -1,28 +1,26 @@
 /* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevpdf.c,v $ $Revision: 1.21.2.5 $ */
+/*$Id: gdevpdf.c,v 1.38.2.1 2002/02/22 19:45:56 ray Exp $ */
 /* PDF-writing driver */
 #include "memory_.h"
 #include "string_.h"
 #include "unistd_.h"
 #include "gx.h"
+#include "gp.h"			/* for gp_get_realtime */
 #include "gserrors.h"
 #include "gxdevice.h"
 #include "gdevpdfx.h"
@@ -35,11 +33,6 @@
 #define PSDF_VERSION_INITIAL psdf_version_ll3
 #define PDF_COMPATIBILITY_LEVEL_INITIAL 1.3
 
-/* Define the names of the resource types. */
-private const char *const resource_type_names[] = {
-    pdf_resource_type_names
-};
-
 /* Define the size of internal stream buffers. */
 /* (This is not a limitation, it only affects performance.) */
 #define sbuf_size 512
@@ -195,11 +188,15 @@ const gx_device_pdf gs_pdfwrite_device =
  1 /*true*/,			/* ReAssignCharacters */
  1 /*true*/,			/* ReEncodeCharacters */
  1,				/* FirstObjectNumber */
+ 0 /*false*/,			/* is_EPS */
+ {-1, -1},			/* doc_dsc_info */
+ {-1, -1},			/* page_dsc_info */
  0 /*false*/,			/* fill_overprint */
  0 /*false*/,			/* stroke_overprint */
  0,				/* overprint_mode */
  gs_no_id,			/* halftone_id */
  {gs_no_id, gs_no_id, gs_no_id, gs_no_id}, /* transfer_ids */
+ 0,				/* transfer_not_identity */
  gs_no_id,			/* black_generation_id */
  gs_no_id,			/* undercolor_removal_id */
  pdf_compress_none,		/* compression */
@@ -210,6 +207,8 @@ const gx_device_pdf gs_pdfwrite_device =
  0,				/* open_font */
  0 /*false*/,			/* use_open_font */
  0,				/* embedded_encoding_id */
+ -1,				/* max_embedded_code */
+ 0,				/* random_offset */
  0,				/* next_id */
  0,				/* Catalog */
  0,				/* Info */
@@ -227,6 +226,7 @@ const gx_device_pdf gs_pdfwrite_device =
  {{0}},				/* text_rotation */
  0,				/* pages */
  0,				/* num_pages */
+ 1,				/* used_mask */
  {
      {
 	 {0}}},			/* resources */
@@ -294,6 +294,7 @@ pdf_close_files(gx_device_pdf * pdev, int code)
 private void
 pdf_reset_page(gx_device_pdf * pdev)
 {
+    pdev->page_dsc_info = gs_pdfwrite_device.page_dsc_info;
     pdev->contents_id = 0;
     pdf_reset_graphics(pdev);
     pdev->procsets = NoMarks;
@@ -371,6 +372,13 @@ pdf_initialize_ids(gx_device_pdf * pdev)
     pdf_create_named_dict(pdev, NULL, &pdev->Pages, 0L);
 }
 
+#ifdef __DECC
+/* The ansi alias rules are violated in this next routine.  Tell the compiler
+   to ignore this.
+ */
+#pragma optimize save
+#pragma optimize ansi_alias=off
+#endif
 /* Update the color mapping procedures after setting ProcessColorModel. */
 void
 pdf_set_process_color_model(gx_device_pdf * pdev)
@@ -390,6 +398,7 @@ pdf_set_process_color_model(gx_device_pdf * pdev)
     case 4:
 	set_dev_proc(pdev, map_rgb_color, NULL);
 	set_dev_proc(pdev, map_color_rgb, cmyk_8bit_map_color_rgb);
+       /* possible problems with aliassing on next statement */
 	set_dev_proc(pdev, map_cmyk_color, cmyk_8bit_map_cmyk_color);
         color = gx_map_cmyk_color((gx_device *)pdev,
 		      frac2cv(frac_0), frac2cv(frac_0),
@@ -401,6 +410,9 @@ pdf_set_process_color_model(gx_device_pdf * pdev)
     color_set_pure(&pdev->fill_color, color);
     color_set_pure(&pdev->stroke_color, color);
 }
+#ifdef __DECC
+#pragma optimize restore
+#endif
 
 /*
  * Reset the text state parameters to initial values.  This isn't a very
@@ -415,6 +427,7 @@ pdf_reset_text(gx_device_pdf * pdev)
     pdev->text.word_spacing = 0;
     pdev->text.leading = 0;
     pdev->text.use_leading = false;
+    pdev->text.render_mode = 0;
 }
 
 /* Open the device. */
@@ -435,6 +448,7 @@ pdf_open(gx_device * dev)
     if (code < 0)
 	goto fail;
     gdev_vector_init((gx_device_vector *) pdev);
+    pdev->vec_procs = &pdf_vector_procs;
     pdev->fill_options = pdev->stroke_options = gx_path_type_optimize;
     /* Set in_page so the vector routines won't try to call */
     /* any vector implementation procedures. */
@@ -477,6 +491,36 @@ pdf_open(gx_device * dev)
     pdev->open_graphics = 0;
     pdf_reset_page(pdev);
 
+    /*
+     * We don't use rand() for generating subset prefixes, because it isn't
+     * random across runs (always starts at 1).  We don't seed rand() from a
+     * one-time source of randomness, because a library should never assume
+     * it can modify program-global state.  So what we do is generate a
+     * one-time random offset, and combine that with the sequence produced
+     * by rand().
+     */
+    {
+	/*
+	 * If we're on a system that provides /dev/random, that's the best
+	 * source of good random bits.
+	 */
+	FILE *rfile = fopen("/dev/random", "rb");
+
+	if (rfile && fread(&pdev->random_offset, sizeof(pdev->random_offset),
+			   1, rfile) == sizeof(pdev->random_offset)
+	    )
+	    fclose(rfile);
+	else {
+	    /* Hope that the clock is random enough. */
+	    long tm[2];
+
+	    if (rfile)
+		fclose(rfile);
+	    gp_get_realtime(tm);
+	    pdev->random_offset = tm[0] + tm[1];
+	}
+    }
+
     return 0;
   fail:
     return pdf_close_files(pdev, code);
@@ -514,42 +558,33 @@ pdf_dominant_rotation(const pdf_text_rotation_t *ptr)
     return angles[imax];
 }
 
-/*
- * Write and release the Cos objects for page-specific resources.
- * We must write all the objects before freeing any of them, because
- * they might refer to each other.
- */
-private int
-pdf_write_resource_objects(gx_device_pdf *pdev, pdf_resource_type_t rtype)
+/* Print a Rotate command for an orientation specified by a DSC comment. */
+private void
+pdf_print_dsc_rotate(stream *s, const gs_point *pbox, int orient)
 {
-    int j;
+    int ori = orient;
 
-    /* Write objects. */
-    for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-	pdf_resource_t *pres = pdev->resources[rtype].chains[j];
-
-	for (; pres != 0; pres = pres->next)
-	    if (!pres->named && !pres->object->written)
-		cos_write_object(pres->object, pdev);
+    if (pbox->x > pbox->y) {
+	/*
+	 * The page is in landscape format.  Adjust the rotation
+	 * accordingly.
+	 */
+	ori ^= 1;
     }
-
-    /* Free unnamed objects, which can't be used again. */
-    for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-	pdf_resource_t **prev = &pdev->resources[rtype].chains[j];
-	pdf_resource_t *pres;
-
-	while ((pres = *prev) != 0) {
-	    if (pres->named) {	/* named, don't free */
-		prev = &pres->next;
-	    } else {
-		cos_free(pres->object, "pdf_write_resource_objects");
-		pres->object = 0;
-		*prev = pres->next;
-	    }
-	}
+    pprintd1(s, "/Rotate %d", ori * 90);
+}
+private bool
+pdf_print_dsc_orientation(stream *s, const gs_point *pbox,
+			  const pdf_page_dsc_info_t *ppdi)
+{
+    if (ppdi->viewing_orientation >= 0) {
+	pdf_print_dsc_rotate(s, pbox, ppdi->viewing_orientation);
+	return true;
+    } else if (ppdi->orientation >= 0) {
+	pdf_print_dsc_rotate(s, pbox, ppdi->orientation);
+	return true;
     }
-
-    return 0;
+    return false;
 }
 
 /* Close the current page. */
@@ -558,6 +593,7 @@ pdf_close_page(gx_device_pdf * pdev)
 {
     int page_num = ++(pdev->next_page);
     pdf_page_t *page;
+    int code;
 
     /*
      * If the very first page is blank, we need to open the document
@@ -577,86 +613,18 @@ pdf_close_page(gx_device_pdf * pdev)
 
     pdf_page_id(pdev, page_num);
     page = &pdev->pages[page_num - 1];
-    page->MediaBox.x = (int)(pdev->MediaSize[0]);
-    page->MediaBox.y = (int)(pdev->MediaSize[1]);
-    page->procsets = pdev->procsets;
+    page->MediaBox.x = pdev->MediaSize[0];
+    page->MediaBox.y = pdev->MediaSize[1];
     page->contents_id = pdev->contents_id;
-
-    /* Write out any resource dictionaries. */
-
-    {
-	int i;
-
-	for (i = 0; i < resourceFont; ++i) {
-	    bool any = false;
-	    stream *s;
-	    int j;
-
-	    for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-		pdf_resource_t *pres = pdev->resources[i].chains[j];
-
-		for (; pres != 0; pres = pres->next) {
-		    if (pres->used_on_page) {
-			long id = pres->object->id;
-
-			if (!any) {
-			    page->resource_ids[i] = pdf_begin_obj(pdev);
-			    s = pdev->strm;
-			    stream_puts(s, "<<");
-			    any = true;
-			}
-			pprintld2(s, "/R%ld\n%ld 0 R", id, id);
-		    }
-		}
-	    }
-	    if (any) {
-		stream_puts(s, ">>\n");
-		pdf_end_obj(pdev);
-		pdf_write_resource_objects(pdev, i);
-	    }
-	}
-    }
+    /* pdf_store_page_resources sets procsets, resource_ids[]. */
+    code = pdf_store_page_resources(pdev, page);
+    if (code < 0)
+	return code;
 
     /* Write out Functions. */
 
     pdf_write_resource_objects(pdev, resourceFunction);
 
-    /* Record references to just those fonts used on this page. */
-
-    {
-	bool any = false;
-	stream *s;
-	int j;
-
-	for (j = 0; j < NUM_RESOURCE_CHAINS; ++j) {
-	    pdf_font_t **prev =
-		(pdf_font_t **)&pdev->resources[resourceFont].chains[j];
-	    pdf_font_t *font;
-
-	    while ((font = *prev) != 0) {
-		if (font->used_on_page) {
-		    if (!any) {
-			page->fonts_id = pdf_begin_obj(pdev);
-			s = pdev->strm;
-			stream_puts(s, "<<");
-			any = true;
-		    }
-		    pprints1(s, "/%s", font->frname);
-		    pprintld1(s, "\n%ld 0 R", font->object->id);
-		    font->used_on_page = false;
-		}
-		if (font->skip) {
-		    /* The font was already written and freed. */
-		    *prev = font->next;
-		} else
-		    prev = &font->next;
-	    }
-	}
-	if (any) {
-	    stream_puts(s, ">>\n");
-	    pdf_end_obj(pdev);
-	}
-    }
     /*
      * When Acrobat Reader 3 prints a file containing a Type 3 font with a
      * non-standard Encoding, it apparently only emits the subset of the
@@ -681,11 +649,28 @@ pdf_close_page(gx_device_pdf * pdev)
 	    pdev->text_rotation.counts[i] += page->text_rotation.counts[i];
     }
 
+    /* Record information from DSC comments. */
+
+    page->dsc_info = pdev->page_dsc_info;
+    if (page->dsc_info.orientation < 0)
+	page->dsc_info.orientation = pdev->doc_dsc_info.orientation;
+    if (page->dsc_info.bounding_box.p.x >= page->dsc_info.bounding_box.q.x ||
+	page->dsc_info.bounding_box.p.y >= page->dsc_info.bounding_box.q.y
+	)
+	page->dsc_info.bounding_box = pdev->doc_dsc_info.bounding_box;
+
+    /* Finish up. */
+
     pdf_reset_page(pdev);
     return (pdf_ferror(pdev) ? gs_note_error(gs_error_ioerror) : 0);
 }
 
 /* Write the page object. */
+private double
+round_box_coord(floatp xy)
+{
+    return (int)(xy * 100 + 0.5) / 100.0;
+}
 private int
 pdf_write_page(gx_device_pdf *pdev, int page_num)
 {
@@ -695,10 +680,16 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
 
     pdf_open_obj(pdev, page_id);
     s = pdev->strm;
-    pprintd2(s, "<</Type/Page/MediaBox [0 0 %d %d]\n",
-	     page->MediaBox.x, page->MediaBox.y);
-    if (page->text_rotation.Rotate >= 0)
-	pprintd1(s, "/Rotate %d", page->text_rotation.Rotate);
+    pprintg2(s, "<</Type/Page/MediaBox [0 0 %g %g]\n",
+	     round_box_coord(page->MediaBox.x),
+	     round_box_coord(page->MediaBox.y));
+    /*
+     * In decreasing priority order, check for %%PageViewingOrientation,
+     * %%PageOrientation, and AutoRotatePages == /PageByPage.
+     */
+    if (!pdf_print_dsc_orientation(s, &page->MediaBox, &page->dsc_info))
+	if (page->text_rotation.Rotate >= 0)
+	    pprintd1(s, "/Rotate %d", page->text_rotation.Rotate);
     pprintld1(s, "/Parent %ld 0 R\n", pdev->Pages->id);
     stream_puts(s, "/Resources<</ProcSet[/PDF");
     if (page->procsets & ImageB)
@@ -713,14 +704,12 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
     {
 	int i;
 
-	for (i = 0; i < resourceFont; ++i)
+	for (i = 0; i < countof(page->resource_ids); ++i)
 	    if (page->resource_ids[i]) {
-		pprints1(s, "/%s ", resource_type_names[i]);
-		pprintld1(s, "%ld 0 R\n", page->resource_ids[i]);
+		stream_puts(s, pdf_resource_type_names[i]);
+		pprintld1(s, " %ld 0 R\n", page->resource_ids[i]);
 	    }
     }
-    if (page->fonts_id)
-	pprintld1(s, "/Font %ld 0 R\n", page->fonts_id);
     stream_puts(s, ">>\n");
 
     /* Write out the annotations array if any. */
@@ -731,9 +720,13 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
 	COS_FREE(page->Annots, "pdf_write_page(Annots)");
 	page->Annots = 0;
     }
-    if (page->contents_id == 0)
-	stream_puts(s, "/Contents []\n");
-    else
+    /*
+     * The PDF documentation allows, and this code formerly emitted,
+     * a Contents entry whose value was an empty array.  Acrobat Reader
+     * 3 and 4 accept this, but Acrobat Reader 5.0 rejects it.
+     * Fortunately, the Contents entry is optional.
+     */
+    if (page->contents_id != 0)
 	pprintld1(s, "/Contents %ld 0 R\n", page->contents_id);
 
     /* Write any elements stored by pdfmarks. */
@@ -796,6 +789,7 @@ pdf_close(gx_device * dev)
     /* Write the font resources and related resources. */
 
     pdf_write_font_resources(pdev);
+    pdf_write_resource_objects(pdev, resourceCMap);
 
     /* Create the Pages tree. */
 
@@ -812,9 +806,21 @@ pdf_close(gx_device * dev)
 	    pprintld1(s, "%ld 0 R\n", pdev->pages[i].Page->id);
     }
     pprintd1(s, "] /Count %d\n", pdev->next_page);
-    if (pdev->params.AutoRotatePages == arp_All)
-	pprintd1(s, "/Rotate %d\n",
-		 pdf_dominant_rotation(&pdev->text_rotation));
+    /*
+     * In decreasing priority order, check for %%ViewingOrientation,
+     * %%Orientation, and AutoRotatePages == /All.  Use the MediaBox of
+     * the first page to determine the document's native portrait vs.
+     * landscape orientation.
+     */
+    {
+	const pdf_page_t *page = &pdev->pages[0];
+
+	if (!pdf_print_dsc_orientation(s, &page->MediaBox,
+				       &pdev->doc_dsc_info))
+	    if (pdev->params.AutoRotatePages == arp_All)
+		pprintd1(s, "/Rotate %d\n",
+			 pdf_dominant_rotation(&pdev->text_rotation));
+    }
     cos_dict_elements_write(pdev->Pages, pdev);
     stream_puts(s, ">>\n");
     pdf_end_obj(pdev);
