@@ -1,23 +1,20 @@
 /* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevpdfi.c,v $ $Revision: 1.20.2.2 $ */
+/*$Id: gdevpdfi.c,v 1.24.2.1 2002/02/22 19:45:56 ray Exp $ */
 /* Image handling for PDF-writing driver */
 #include "gx.h"
 #include "gserrors.h"
@@ -271,10 +268,8 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
     in_line &= nbytes <= MAX_INLINE_IMAGE_BYTES;
     if (rect.p.x != 0 || rect.p.y != 0 ||
 	rect.q.x != pim->Width || rect.q.y != pim->Height ||
-	(is_mask ? pim->CombineWithColor :
-	 pdf_color_space(pdev, &cs_value, pcs,
-			 (in_line ? &pdf_color_space_names_short :
-			  &pdf_color_space_names), in_line) < 0)
+	(is_mask && pim->CombineWithColor)
+	/* Color space setup used to be done here: see SRZB comment below. */
 	) {
 	gs_free_object(mem, pie, "pdf_begin_image");
 	goto nyi;
@@ -298,15 +293,38 @@ pdf_begin_typed_image(gx_device_pdf *pdev, const gs_imager_state * pis,
     }
     if ((code = pdf_begin_write_image(pdev, &pie->writer, gs_no_id, width,
 				      height, NULL, in_line)) < 0 ||
-	/****** pctm IS WRONG ******/
-	(code = psdf_setup_image_filters((gx_device_psdf *) pdev,
-					 &pie->writer.binary, &image.pixel,
-					 pmat, pis)) < 0 ||
+	/*
+	 * Some regrettable PostScript code (such as LanguageLevel 1 output
+	 * from Microsoft's PSCRIPT.DLL driver) misuses the transfer
+	 * function to accomplish the equivalent of indexed color.
+	 * Downsampling (well, only averaging) or JPEG compression are not
+	 * compatible with this.  Play it safe by using only lossless
+	 * filters if the transfer function(s) is/are other than the
+	 * identity.
+	 */
+	(code = (pdev->transfer_not_identity ?
+		 psdf_setup_lossless_filters((gx_device_psdf *) pdev,
+					     &pie->writer.binary,
+					     &image.pixel) :
+		 psdf_setup_image_filters((gx_device_psdf *) pdev,
+					  &pie->writer.binary, &image.pixel,
+					  pmat, pis))) < 0 ||
+	/* SRZB 2001-04-25/Bl
+	 * Since psdf_setup_image_filters may change the color space
+	 * (in case of pdev->params.ConvertCMYKImagesToRGB == true),
+	 * we postpone the selection of the PDF color space to here:
+	 */
+	(!is_mask &&
+	 (code = pdf_color_space(pdev, &cs_value, image.pixel.ColorSpace,
+				 (in_line ? &pdf_color_space_names_short :
+				  &pdf_color_space_names), in_line)) < 0) ||
 	(code = pdf_begin_image_data(pdev, &pie->writer,
 				     (const gs_pixel_image_t *)&image,
 				     &cs_value)) < 0
-	)
-	return code;
+	) {
+	/****** SHOULD FREE STRUCTURES AND CLEAN UP HERE ******/
+	goto nyi;		/* fall back to default implementation */
+    }
     return 0;
  nyi:
     return gx_default_begin_typed_image
