@@ -1,23 +1,20 @@
 /* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevpng.c,v $ $Revision: 1.3.2.1 $ */
+/*$Id: gdevpng.c,v 1.4.2.1 2002/02/22 19:45:56 ray Exp $ */
 /* PNG (Portable Network Graphics) Format.  Pronounced "ping". */
 /* lpd 1999-09-24: changes PNG_NO_STDIO to PNG_NO_CONSOLE_IO for libpng
    versions 1.0.3 and later. */
@@ -82,12 +79,13 @@ prn_device(prn_std_procs, "pngmono",
 private const gx_device_procs png16_procs =
 prn_color_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
 		pc_4bit_map_rgb_color, pc_4bit_map_color_rgb);
-const gx_device_printer gs_png16_device =
-prn_device(png16_procs, "png16",
+const gx_device_printer gs_png16_device = {
+  prn_device_body(gx_device_printer, png16_procs, "png16",
 	   DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
 	   X_DPI, Y_DPI,
 	   0, 0, 0, 0,		/* margins */
-	   4, png_print_page);
+	   3, 4, 3, 2, 4, 3, png_print_page)
+};
 
 /* 8-bit (SuperVGA-style) color. */
 /* (Uses a fixed palette of 3,3,2 bits.) */
@@ -95,12 +93,13 @@ prn_device(png16_procs, "png16",
 private const gx_device_procs png256_procs =
 prn_color_procs(gdev_prn_open, gdev_prn_output_page, gdev_prn_close,
 		pc_8bit_map_rgb_color, pc_8bit_map_color_rgb);
-const gx_device_printer gs_png256_device =
-prn_device(png256_procs, "png256",
+const gx_device_printer gs_png256_device = {
+  prn_device_body(gx_device_printer, png256_procs, "png256",
 	   DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS,
 	   X_DPI, Y_DPI,
 	   0, 0, 0, 0,		/* margins */
-	   8, png_print_page);
+	   3, 8, 6, 6, 7, 7, png_print_page)
+};
 
 /* 8-bit gray */
 
