@@ -1,23 +1,20 @@
 /* Copyright (C) 1997, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevpx.c,v $ $Revision: 1.4.2.1 $ */
+/*$Id: gdevpx.c,v 1.6.2.1 2002/02/22 19:45:56 ray Exp $ */
 /* H-P PCL XL driver */
 #include "math_.h"
 #include "memory_.h"
@@ -258,6 +255,7 @@ pclxl_set_color(gx_device_pclxl * xdev, const gx_drawing_color * pdc,
 }
 
 /* Test whether we can handle a given color space in an image. */
+/* We cannot handle ICCBased color spaces. */
 private bool
 pclxl_can_handle_color_space(const gs_color_space * pcs)
 {
@@ -270,7 +268,8 @@ pclxl_can_handle_color_space(const gs_color_space * pcs)
 	    gs_color_space_get_index(gs_color_space_indexed_base_space(pcs));
     }
     return !(index == gs_color_space_index_Separation ||
-	     index == gs_color_space_index_Pattern);
+	     index == gs_color_space_index_Pattern ||
+             index == gs_color_space_index_CIEICC);
 }
 
 /* Set brush, pen, and mode for painting a path. */
@@ -1097,10 +1096,10 @@ pclxl_open_device(gx_device * dev)
     gx_device_pclxl *const xdev = (gx_device_pclxl *)dev;
     int code;
 
-    vdev->v_memory = dev->memory;
-/****** WRONG ******/
+    vdev->v_memory = dev->memory;	/****** WRONG ******/
     vdev->vec_procs = &pclxl_vector_procs;
-    code = gdev_vector_open_file(vdev, 512);
+    code = gdev_vector_open_file_options(vdev, 512,
+					 VECTOR_OPEN_FILE_SEQUENTIAL);
     if (code < 0)
 	return code;
     pclxl_page_init(xdev);
