@@ -1,23 +1,20 @@
 /* Copyright (C) 1992, 1995, 1996, 1997, 1999 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevwdib.c,v $ $Revision: 1.2.2.1 $ */
+/*$Id: gdevwdib.c,v 1.3.2.1 2002/02/22 19:45:57 ray Exp $ */
 /* MS Windows 3.n driver for Ghostscript using a DIB for buffering. */
 #include "gdevmswn.h"
 #include "gxdevmem.h"
@@ -141,10 +138,12 @@ win_dib_open(gx_device * dev)
 	return code;
     }
     /* notify caller about new device */
-    (*pgsdll_callback) (GSDLL_DEVICE, (unsigned char *)dev, 1);
-    (*pgsdll_callback) (GSDLL_SIZE, (unsigned char *)dev,
+    if (pgsdll_callback) {
+	(*pgsdll_callback) (GSDLL_DEVICE, (unsigned char *)dev, 1);
+	(*pgsdll_callback) (GSDLL_SIZE, (unsigned char *)dev,
 			(dev->width & 0xffff) +
 			((ulong) (dev->height & 0xffff) << 16));
+    }
     return code;
 }
 
@@ -169,7 +168,8 @@ win_dib_close(gx_device * dev)
 
     /* wait until bitmap is not being used by caller */
     win_dib_lock_device((unsigned char *)dev, 1);
-    (*pgsdll_callback) (GSDLL_DEVICE, (unsigned char *)dev, 0);
+    if (pgsdll_callback)
+	(*pgsdll_callback) (GSDLL_DEVICE, (unsigned char *)dev, 0);
     win_dib_lock_device((unsigned char *)dev, 0);
     win_dib_free_bitmap((gx_device_win *) dev);
 #ifdef __WIN32__
@@ -605,7 +605,7 @@ win_dib_alloc_bitmap(gx_device_win * dev, gx_device * param_dev)
     wdev->mdev.base = (byte *) base;
     wmproc(open_device) ((gx_device *) & wdev->mdev);
 
-    if (wdev->is_open)
+    if (wdev->is_open && pgsdll_callback)
 	(*pgsdll_callback) (GSDLL_SIZE, (unsigned char *)dev,
 			    (dev->width & 0xffff) +
 			    ((ulong) (dev->height & 0xffff) << 16));
