@@ -1,23 +1,20 @@
 /* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gdevwpr2.c,v $ $Revision: 1.5.2.1 $ */
+/*$Id: gdevwpr2.c,v 1.9.2.1 2002/02/22 19:45:57 ray Exp $ */
 /*
  * Microsoft Windows 3.n printer driver for Ghostscript.
  * Original version by Russell Lang and
@@ -30,6 +27,8 @@
  * Bug fixed by Pierre Arnaud 2000-03-09 (win_pr2_put_params error when is_open)
  * Bug fixed by Pierre Arnaud 2000-03-20 (win_pr2_set_bpp did not set anti_alias)
  * Bug fixed by Pierre Arnaud 2000-03-22 (win_pr2_set_bpp depth was wrong)
+ * Modified by Pierre Arnaud 2000-12-12 (mainly added support for Tumble)
+ * Bug fixed by Pierre Arnaud 2000-12-18 (-dQueryUser now works from cmd line)
  */
 
 /* This driver uses the printer default size and resolution and
@@ -47,7 +46,7 @@
  * rjl 1997-11-20
  */
 
-/* Additions by Pierre Arnaud (Pierre.Arnaud@iname.com) 1992-11-20
+/* Additions by Pierre Arnaud (Pierre.Arnaud@opac.ch)
  *
  * The driver has been extended in order to provide some run-time
  * feed-back about the default Windows printer and to give the user
@@ -129,6 +128,7 @@
 #include "gp_mswin.h"
 
 #include "gp.h"
+#include "gpcheck.h"
 #include "commdlg.h"
 
 
@@ -156,6 +156,10 @@ prn_color_params_procs(win_pr2_open, gdev_prn_output_page, win_pr2_close,
 		       win_pr2_map_rgb_color, win_pr2_map_color_rgb,
 		       win_pr2_get_params, win_pr2_put_params);
 
+#define PARENT_WINDOW  HWND_DESKTOP
+BOOL CALLBACK CancelDlgProc(HWND, UINT, WPARAM, LPARAM);
+BOOL CALLBACK AbortProc2(HDC, int);
+
 
 /* The device descriptor */
 typedef struct gx_device_win_pr2_s gx_device_win_pr2;
@@ -181,13 +185,17 @@ struct gx_device_win_pr2_s {
     int max_dpi;		/* maximum resolution in DPI */
     int ratio;			/* stretch ratio when printing */
     int selected_bpp;		/* selected bpp, memorised by win_pr2_set_bpp */
+    bool tumble;		/* tumble setting (with duplex) */
+    int query_user;		/* query user (-dQueryUser) */
 
     HANDLE win32_hdevmode;	/* handle to device mode information */
     HANDLE win32_hdevnames;	/* handle to device names information */
 
     DLGPROC lpfnAbortProc;
     DLGPROC lpfnCancelProc;
+    HWND hDlgModeless;
 
+    bool use_old_spool_name;	/* user prefers old \\spool\ name */
     gx_device_win_pr2* original_device;	/* used to detect copies */
 };
 
@@ -215,10 +223,13 @@ gx_device_win_pr2 far_data gs_mswinpr2_device =
     0,				/* max_dpi */
     0,				/* ratio */
     0,				/* selected_bpp */
+    false,			/* tumble */
+    -1,				/* query_user */
     NULL,			/* win32_hdevmode */
     NULL,			/* win32_hdevnames */
     NULL,			/* lpfnAbortProc */
     NULL,			/* lpfnCancelProc */
+    false,			/* use_old_spool_name */
     NULL			/* original_device */
 };
 
@@ -250,12 +261,6 @@ win_pr2_open(gx_device * dev)
     
     win_pr2_copy_check(wdev);
 
-    if ((!wdev->nocancel) && (hDlgModeless)) {
-	/* device cannot opened twice since only one hDlgModeless */
-	fprintf(stderr, "Can't open mswinpr2 device twice\n");
-	return gs_error_limitcheck;
-    }
-
     /* get a HDC for the printer */
     if ((wdev->win32_hdevmode) &&
 	(wdev->win32_hdevnames)) {
@@ -286,7 +291,7 @@ win_pr2_open(gx_device * dev)
 	memset(&pd, 0, sizeof(pd));
 	
 	pd.lStructSize = sizeof(pd);
-	pd.hwndOwner = hwndtext;
+	pd.hwndOwner = PARENT_WINDOW;
 	pd.Flags = PD_RETURNDC;
 	pd.nMinPage = wdev->doc_page_begin;
 	pd.nMaxPage = wdev->doc_page_end;
@@ -316,20 +321,12 @@ win_pr2_open(gx_device * dev)
 	pd.hDevNames = NULL;
     }
     if (!(GetDeviceCaps(wdev->hdcprn, RASTERCAPS) != RC_DIBTODEV)) {
-	fprintf(stderr, "Windows printer does not have RC_DIBTODEV\n");
+	errprintf( "Windows printer does not have RC_DIBTODEV\n");
 	DeleteDC(wdev->hdcprn);
 	return gs_error_limitcheck;
     }
     /* initialise printer, install abort proc */
-#ifdef __WIN32__
-    wdev->lpfnAbortProc = (DLGPROC) AbortProc;
-#else
-#ifdef __DLL__
-    wdev->lpfnAbortProc = (DLGPROC) GetProcAddress(phInstance, "AbortProc");
-#else
-    wdev->lpfnAbortProc = (DLGPROC) MakeProcInstance((FARPROC) AbortProc, phInstance);
-#endif
-#endif
+    wdev->lpfnAbortProc = (DLGPROC) AbortProc2;
     SetAbortProc(wdev->hdcprn, (ABORTPROC) wdev->lpfnAbortProc);
 
     /*
@@ -349,10 +346,7 @@ win_pr2_open(gx_device * dev)
     }
 
     if (StartDoc(wdev->hdcprn, &docinfo) <= 0) {
-	fprintf(stderr, "Printer StartDoc failed (error %08x)\n", GetLastError());
-#if !defined(__WIN32__) && !defined(__DLL__)
-	FreeProcInstance((FARPROC) wdev->lpfnAbortProc);
-#endif
+	errprintf("Printer StartDoc failed (error %08x)\n", GetLastError());
 	DeleteDC(wdev->hdcprn);
 	return gs_error_limitcheck;
     }
@@ -409,18 +403,10 @@ win_pr2_open(gx_device * dev)
 
     if (!wdev->nocancel) {
 	/* inform user of progress with dialog box and allow cancel */
-#ifdef __WIN32__
 	wdev->lpfnCancelProc = (DLGPROC) CancelDlgProc;
-#else
-#ifdef __DLL__
-	wdev->lpfnCancelProc = (DLGPROC) GetProcAddress(phInstance, "CancelDlgProc");
-#else
-	wdev->lpfnCancelProc = (DLGPROC) MakeProcInstance((FARPROC) CancelDlgProc, phInstance);
-#endif
-#endif
-	hDlgModeless = CreateDialog(phInstance, "CancelDlgBox",
-				    hwndtext, wdev->lpfnCancelProc);
-	ShowWindow(hDlgModeless, SW_HIDE);
+	wdev->hDlgModeless = CreateDialog(phInstance, "CancelDlgBox",
+				    PARENT_WINDOW, wdev->lpfnCancelProc);
+	ShowWindow(wdev->hDlgModeless, SW_HIDE);
     }
     return code;
 };
@@ -437,23 +423,17 @@ win_pr2_close(gx_device * dev)
     /* Free resources */
 
     if (!wdev->nocancel) {
-	if (!hDlgModeless)
+	if (!wdev->hDlgModeless)
 	    aborted = TRUE;
 	else
-	    DestroyWindow(hDlgModeless);
-	hDlgModeless = 0;
-#if !defined(__WIN32__) && !defined(__DLL__)
-	FreeProcInstance((FARPROC) wdev->lpfnCancelProc);
-#endif
+	    DestroyWindow(wdev->hDlgModeless);
+	wdev->hDlgModeless = 0;
     }
     if (aborted)
 	AbortDoc(wdev->hdcprn);
     else
 	EndDoc(wdev->hdcprn);
 
-#if !defined(__WIN32__) && !defined(__DLL__)
-    FreeProcInstance((FARPROC) wdev->lpfnAbortProc);
-#endif
     DeleteDC(wdev->hdcprn);
 
     if (wdev->win32_hdevmode != NULL) {
@@ -558,8 +538,8 @@ win_pr2_print_page(gx_device_printer * pdev, FILE * file)
 
     if (!wdev->nocancel) {
 	sprintf(dlgtext, "Printing page %d", (int)(pdev->PageCount) + 1);
-	SetWindowText(GetDlgItem(hDlgModeless, CANCEL_PRINTING), dlgtext);
-	ShowWindow(hDlgModeless, SW_SHOW);
+	SetWindowText(GetDlgItem(wdev->hDlgModeless, CANCEL_PRINTING), dlgtext);
+	ShowWindow(wdev->hDlgModeless, SW_SHOW);
     }
     for (y = 0; y < scan_lines;) {
 	/* copy slice to row buffer */
@@ -587,31 +567,31 @@ win_pr2_print_page(gx_device_printer * pdev, FILE * file)
 	if (!wdev->nocancel) {
 	    /* inform user of progress */
 	    sprintf(dlgtext, "%d%% done", (int)(y * 100L / scan_lines));
-	    SetWindowText(GetDlgItem(hDlgModeless, CANCEL_PCDONE), dlgtext);
+	    SetWindowText(GetDlgItem(wdev->hDlgModeless, CANCEL_PCDONE), dlgtext);
 	}
 	/* process message loop */
-	while (PeekMessage(&msg, hDlgModeless, 0, 0, PM_REMOVE)) {
-	    if ((hDlgModeless == 0) || !IsDialogMessage(hDlgModeless, &msg)) {
+	while (PeekMessage(&msg, wdev->hDlgModeless, 0, 0, PM_REMOVE)) {
+	    if ((wdev->hDlgModeless == 0) || !IsDialogMessage(wdev->hDlgModeless, &msg)) {
 		TranslateMessage(&msg);
 		DispatchMessage(&msg);
 	    }
 	}
-	if ((!wdev->nocancel) && (hDlgModeless == 0)) {
+	if ((!wdev->nocancel) && (wdev->hDlgModeless == 0)) {
 	    /* user pressed cancel button */
 	    break;
 	}
     }
 
-    if ((!wdev->nocancel) && (hDlgModeless == 0))
+    if ((!wdev->nocancel) && (wdev->hDlgModeless == 0))
 	code = gs_error_Fatal;	/* exit Ghostscript cleanly */
     else {
 	/* push out the page */
 	if (!wdev->nocancel)
-	    SetWindowText(GetDlgItem(hDlgModeless, CANCEL_PCDONE),
+	    SetWindowText(GetDlgItem(wdev->hDlgModeless, CANCEL_PCDONE),
 			  "Ejecting page...");
 	EndPage(wdev->hdcprn);
 	if (!wdev->nocancel)
-	    ShowWindow(hDlgModeless, SW_HIDE);
+	    ShowWindow(wdev->hDlgModeless, SW_HIDE);
     }
 
   bmp_done:
@@ -722,8 +702,15 @@ win_pr2_get_params(gx_device * pdev, gs_param_list * plist)
     if (code >= 0)
 	code = param_write_bool(plist, "NoCancel",
 				&(wdev->nocancel));
+    if (code >= 0)
+	code = param_write_bool(plist, "QueryUser",
+				&(wdev->query_user));
     if (code >= 0)
 	code = win_pr2_write_user_settings(wdev, plist);
+    
+    if ((code >= 0) && (wdev->Duplex_set > 0))
+	code = param_write_bool(plist, "Tumble",
+				&(wdev->tumble));
 
     return code;
 }
@@ -737,9 +724,11 @@ win_pr2_put_params(gx_device * pdev, gs_param_list * plist)
     int ecode = 0, code;
     int old_bpp = pdev->color_info.depth;
     int bpp = old_bpp;
+    bool tumble   = wdev->tumble;
     bool nocancel = wdev->nocancel;
     int queryuser = 0;
     bool old_duplex = wdev->Duplex;
+    bool old_tumble = wdev->tumble;
     int  old_orient = wdev->user_orient;
     int  old_color  = wdev->user_color;
     int  old_paper  = wdev->user_paper;
@@ -748,6 +737,7 @@ win_pr2_put_params(gx_device * pdev, gs_param_list * plist)
     if (wdev->Duplex_set < 0) {
 	wdev->Duplex_set = 0;
 	wdev->Duplex = false;
+	wdev->tumble = false;
     }
     
     win_pr2_copy_check(wdev);
@@ -792,6 +782,17 @@ win_pr2_put_params(gx_device * pdev, gs_param_list * plist)
 	    break;
     }
 
+    switch (code = param_read_bool(plist, "Tumble", &tumble)) {
+	case 0:
+	    wdev->tumble = tumble;
+	    break;
+	default:
+	    ecode = code;
+	    param_signal_error(plist, "Tumble", ecode);
+	case 1:
+	    break;
+    }
+
     switch (code = param_read_int(plist, "QueryUser", &queryuser)) {
 	case 0:
 	    if ((queryuser > 0) &&
@@ -811,6 +812,7 @@ win_pr2_put_params(gx_device * pdev, gs_param_list * plist)
     
     if (wdev->win32_hdevmode && wdev->hdcprn) {
 	if ( (old_duplex != wdev->Duplex)
+	  || (old_tumble != wdev->tumble)
 	  || (old_orient != wdev->user_orient)
 	  || (old_color  != wdev->user_color)
 	  || (old_paper  != wdev->user_paper)
@@ -833,9 +835,6 @@ win_pr2_put_params(gx_device * pdev, gs_param_list * plist)
 
 /********************************************************************************/
 
-#ifndef __WIN32__
-#include <print.h>
-#endif
 
 /* Get Device Context for printer */
 private int
@@ -864,16 +863,19 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     LPFNDEVCAPS pfnDeviceCapabilities;
     LPDEVMODE podevmode, pidevmode;
 
-#ifdef __WIN32__
     HANDLE hprinter;
-#endif
 
     /* first try to derive the printer name from -sOutputFile= */
-    /* is printer if name prefixed by \\spool\ */
-    if (is_spool(wdev->fname))
+    /* is printer if name prefixed by \\spool\ or by %printer% */
+    if (is_spool(wdev->fname)) {
 	device = wdev->fname + 8;	/* skip over \\spool\ */
-    else
+	wdev->use_old_spool_name = true;
+    } else if (strncmp("%printer%",wdev->fname,9) == 0) {
+	device = wdev->fname + 9;	/* skip over %printer% */
+	wdev->use_old_spool_name = false;
+    } else {
 	return FALSE;
+    }
 
     /* now try to match the printer name against the [Devices] section */
     if ((devices = gs_malloc(4096, 1, "win_pr2_getdc")) == (char *)NULL)
@@ -895,15 +897,12 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     GetProfileString("Devices", device, "", driverbuf, sizeof(driverbuf));
     driver = strtok(driverbuf, ",");
     output = strtok(NULL, ",");
-#ifdef __WIN32__
     if (is_win32s)
-#endif
     {
 	strcpy(drvname, driver);
 	strcat(drvname, ".drv");
 	driver = drvname;
     }
-#ifdef __WIN32__
 
     if (!is_win32s) {		/* Win32 */
 	if (!OpenPrinter(device, &hprinter, NULL))
@@ -921,7 +920,6 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
 	DocumentProperties(NULL, hprinter, device, podevmode, NULL, DM_OUT_BUFFER);
 	pfnDeviceCapabilities = (LPFNDEVCAPS) DeviceCapabilities;
     } else
-#endif
     {				/* Win16 and Win32s */
 	/* now load the printer driver */
 	hlib = LoadLibrary(driver);
@@ -1055,7 +1053,6 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
     
     win_pr2_update_win(wdev, pidevmode);
     
-#ifdef WIN32
     if (!is_win32s) {
 	
 	/* merge the entries */
@@ -1065,7 +1062,6 @@ win_pr2_getdc(gx_device_win_pr2 * wdev)
 	/* now get a DC */
 	wdev->hdcprn = CreateDC(driver, device, NULL, podevmode);
     } else
-#endif
     {				/* Win16 and Win32s */
 	pfnExtDeviceMode(NULL, hlib, podevmode, device, output,
 			 pidevmode, NULL, DM_IN_BUFFER | DM_OUT_BUFFER);
@@ -1126,6 +1122,7 @@ win_pr2_update_dev(gx_device_win_pr2 * dev, LPDEVMODE pdevmode)
     if (pdevmode->dmFields & DM_DUPLEX) {
 	dev->Duplex_set = 1;
 	dev->Duplex = pdevmode->dmDuplex == DMDUP_SIMPLEX ? false : true;
+	dev->tumble = pdevmode->dmDuplex == DMDUP_HORIZONTAL ? true : false;
     }
     
     return TRUE;
@@ -1138,9 +1135,9 @@ win_pr2_update_win(gx_device_win_pr2 * dev, LPDEVMODE pdevmode)
 	pdevmode->dmFields |= DM_DUPLEX;
 	pdevmode->dmDuplex = DMDUP_SIMPLEX;
 	if (dev->Duplex) {
-/*	    if (dev->Tumble) {
+	    if (dev->tumble == false) {
 		pdevmode->dmDuplex = DMDUP_VERTICAL;
-	    } else */ {
+	    } else {
 		pdevmode->dmDuplex = DMDUP_HORIZONTAL;
 	    }
 	}
@@ -1160,6 +1157,7 @@ win_pr2_update_win(gx_device_win_pr2 * dev, LPDEVMODE pdevmode)
 	pdevmode->dmFields |= DM_PAPERSIZE;
 	pdevmode->dmPaperSize = dev->user_paper;
     }
+    return 0;
 }
 
 /********************************************************************************/
@@ -1365,10 +1363,11 @@ win_pr2_print_setup_interaction(gx_device_win_pr2 * wdev, int mode)
     LPDEVNAMES devnames;
 
     wdev->user_changed_settings = FALSE;
+    wdev->query_user = mode;
 
     memset(&pd, 0, sizeof(pd));
     pd.lStructSize = sizeof(pd);
-    pd.hwndOwner = hwndtext;
+    pd.hwndOwner = PARENT_WINDOW;
 
     switch (mode) {
 	case 2:	pd.Flags = PD_PRINTSETUP; break;
@@ -1394,7 +1393,11 @@ win_pr2_print_setup_interaction(gx_device_win_pr2 * wdev, int mode)
     devnames = (LPDEVNAMES) GlobalLock(pd.hDevNames);
 
     wdev->user_changed_settings = TRUE;
-    sprintf(wdev->fname, "\\\\spool\\%s", (char*)(devnames)+(devnames->wDeviceOffset));
+    if (wdev->use_old_spool_name) {
+	sprintf(wdev->fname, "\\\\spool\\%s", (char*)(devnames)+(devnames->wDeviceOffset));
+    } else {
+	sprintf(wdev->fname, "%%printer%%%s", (char*)(devnames)+(devnames->wDeviceOffset));
+    }
 
     if (mode == 3) {
 	devmode->dmCopies = wdev->user_copies * wdev->print_copies;
@@ -1414,6 +1417,7 @@ win_pr2_print_setup_interaction(gx_device_win_pr2 * wdev, int mode)
     if (devmode->dmFields & DM_DUPLEX) {
 	wdev->Duplex_set = 1;
 	wdev->Duplex = devmode->dmDuplex == DMDUP_SIMPLEX ? false : true;
+	wdev->tumble = devmode->dmDuplex == DMDUP_HORIZONTAL ? true : false;
     }
 
     {
@@ -1493,3 +1497,34 @@ win_pr2_copy_check(gx_device_win_pr2 * wdev)
 	}
     }
 }
+
+
+/* Modeless dialog box - Cancel printing */
+BOOL CALLBACK 
+CancelDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
+{
+    switch (message) {
+	case WM_INITDIALOG:
+	    SetWindowText(hDlg, szAppName);
+	    return TRUE;
+	case WM_COMMAND:
+	    switch (LOWORD(wParam)) {
+		case IDCANCEL:
+		    DestroyWindow(hDlg);
+		    EndDialog(hDlg, 0);
+		    return TRUE;
+	    }
+    }
+    return FALSE;
+}
+
+
+BOOL CALLBACK 
+AbortProc2(HDC hdcPrn, int code)
+{
+    process_interrupts();
+    if (code == SP_OUTOFDISK)
+	return (FALSE);		/* cancel job */
+    return (TRUE);
+}
+
