@@ -1,23 +1,20 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2000, 2001 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gsfunc4.c,v $ $Revision: 1.3.2.1 $ */
+/*$Id: gsfunc4.c,v 1.7.2.1 2002/02/22 19:45:57 ray Exp $ */
 /* Implementation of FunctionType 4 (PostScript Calculator) Functions */
 #include "math_.h"
 #include "memory_.h"
@@ -107,7 +104,8 @@ private int
 fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 {
     const gs_function_PtCr_t *pfn = (const gs_function_PtCr_t *)pfn_common;
-    calc_value_t vstack[1 + MAX_VSTACK + 1];
+    calc_value_t vstack_buf[2 + MAX_VSTACK + 1];
+    calc_value_t *vstack = &vstack_buf[1];
     calc_value_t *vsp = vstack + pfn->params.m;
     const byte *p = pfn->params.ops.data;
     int i;
@@ -216,6 +214,7 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 
     };
 
+    vstack[-1].type = CVT_NONE;  /* for type dispatch in empty stack case */
     vstack[0].type = CVT_NONE;	/* catch underflow */
     for (i = 0; i < pfn->params.m; ++i)
 	store_float(&vstack[i + 1], in[i]);
@@ -292,7 +291,7 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 	    vsp->value.f = ceil(vsp->value.f);
 	    continue;
 	case PtCr_cos:
-	    vsp->value.f = cos(vsp->value.f);
+	    vsp->value.f = gs_cos_degrees(vsp->value.f);
 	    continue;
 	case PtCr_cvi:
 	    vsp->value.i = (int)(vsp->value.f);
@@ -365,7 +364,7 @@ fn_PtCr_evaluate(const gs_function_t *pfn_common, const float *in, float *out)
 	    vsp->value.f = floor(vsp->value.f + 0.5);
 	    continue;
 	case PtCr_sin:
-	    vsp->value.f = sin(vsp->value.f);
+	    vsp->value.f = gs_sin_degrees(vsp->value.f);
 	    continue;
 	case PtCr_sqrt:
 	    vsp->value.f = sqrt(vsp->value.f);
@@ -586,10 +585,10 @@ calc_put_ops(stream *s, const byte *ops, uint size)
 	    break;
 	}
 	case PtCr_true:
-	    pputs(s, "true ");
+	    stream_puts(s, "true ");
 	    break;
 	case PtCr_false:
-	    pputs(s, "false ");
+	    stream_puts(s, "false ");
 	    break;
 	case PtCr_if: {
 	    int skip = (p[0] << 8) + p[1];
@@ -605,13 +604,15 @@ calc_put_ops(stream *s, const byte *ops, uint size)
 		p += skip;
 		if (code < 0)
 		    return code;
-		pputs(s, " ifelse ");
+		stream_puts(s, " ifelse ");
 	    } else
-		pputs(s, " if ");
+		stream_puts(s, " if ");
+	    break;
 	}
 	case PtCr_else:
 	    if (p != ops + size - 2)
 		return_error(gs_error_rangecheck);
+	    spputc(s, '}');
 	    return 1;
 	/*case PtCr_return:*/	/* not possible */
 	default: {		/* must be < PtCr_NUM_OPS */
@@ -647,18 +648,30 @@ calc_access(const gs_data_source_t *psrc, ulong start, uint length,
     const gs_function_PtCr_t *const pfn =
 	(const gs_function_PtCr_t *)
 	  ((const char *)psrc - offset_of(gs_function_PtCr_t, data_source));
+    /*
+     * The caller wants a specific substring of the symbolic definition.
+     * Generate the entire definition, using a SubFileDecode filter (in an
+     * output pipeline!) to extract the substring.  This is very
+     * inefficient, but this code is rarely used, and almost never actually
+     * has to break up the definition into pieces to fit in the caller's
+     * buffer.
+     */
     stream_SFD_state st;
-    stream s;
+    stream ds, bs;
+    byte dbuf[200];		/* arbitrary */
     const stream_template *const template = &s_SFD_template;
 
+    /* Set up the stream that writes into the buffer. */
+    s_init(&bs, NULL);
+    swrite_string(&bs, buf, length);
+    /* Set up the SubFileDecode stream. */
+    s_init(&ds, NULL);
+    s_init_state((stream_state *)&st, template, NULL);
     template->set_defaults((stream_state *)&st);
     st.skip_count = start;
-    swrite_string(&s, buf, length);
-    s.procs.process = template->process;
-    s.state = (stream_state *)&st;
-    if (template->init)
-	template->init((stream_state *)&st);
-    calc_put(&s, pfn);
+    s_init_filter(&ds, (stream_state *)&st, dbuf, sizeof(dbuf), &bs);
+    calc_put(&ds, pfn);
+    sclose(&ds);
     if (ptr)
 	*ptr = buf;
     return 0;
