@@ -1,23 +1,20 @@
 /* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gslib.c,v $ $Revision: 1.3.2.1 $ */
+/*$Id: gslib.c,v 1.5.2.1 2002/02/22 19:45:58 ray Exp $ */
 /* Test program for Ghostscript library */
 /* Capture stdin/out/err before gsio.h redefines them. */
 #include "stdio_.h"
@@ -93,6 +90,31 @@ extern_gs_lib_device_list();
 /* Forward references */
 private float odsf(P2(floatp, floatp));
 
+/* Provide a single point for all "C" stdout and stderr.
+ * Eventually these will always be referenced through an instance structure. 
+ */
+
+int outwrite(const char *str, int len)
+{
+    return fwrite(str, 1, len, gs_stdout);
+}
+
+int errwrite(const char *str, int len)
+{
+    return fwrite(str, 1, len, gs_stderr);
+}
+
+void outflush()
+{
+    fflush(gs_stdout);
+}
+
+void errflush()
+{
+    fflush(gs_stderr);
+}
+
+
 int
 main(int argc, const char *argv[])
 {
@@ -213,7 +235,7 @@ main(int argc, const char *argv[])
     if (code)
 	dprintf1("**** Test returned code = %d.\n", code);
     dputs("Done.  Press <enter> to exit.");
-    getchar();
+    fgetc(gs_stdin);
     gs_lib_finit(0, 0);
     return 0;
 #undef mem
