@@ -1,23 +1,20 @@
 /* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gsstate.c,v $ $Revision: 1.5.2.4 $ */
+/*$Id: gsstate.c,v 1.11.2.1 2002/02/22 19:45:58 ray Exp $ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -137,7 +134,6 @@ private int gstate_copy(P4(gs_state *, const gs_state *,
  * global gstates to local objects).
  */
 
-
 /*
  * Define these elements of the graphics state that are allocated
  * individually for each state, except for line_params.dash.pattern.
@@ -164,11 +160,13 @@ public_st_gs_state();
 
 /* GC procedures for gs_state */
 private ENUM_PTRS_WITH(gs_state_enum_ptrs, gs_state *gsvptr)
-ENUM_PREFIX(st_imager_state, gs_state_num_ptrs + 1);
+ENUM_PREFIX(st_imager_state, gs_state_num_ptrs + 2);
 #define e1(i,elt) ENUM_PTR(i,gs_state,elt);
 gs_state_do_ptrs(e1)
 case gs_state_num_ptrs:	/* handle device specially */
 ENUM_RETURN(gx_device_enum_ptr(gsvptr->device));
+case gs_state_num_ptrs + 1:	/* handle device filter stack specially */
+ENUM_RETURN(gsvptr->dfilter_stack);
 #undef e1
 ENUM_PTRS_END
 private RELOC_PTRS_WITH(gs_state_reloc_ptrs, gs_state *gsvptr)
@@ -179,6 +177,7 @@ private RELOC_PTRS_WITH(gs_state_reloc_ptrs, gs_state *gsvptr)
 	gs_state_do_ptrs(r1)
 #undef r1
 	gsvptr->device = gx_device_reloc_ptr(gsvptr->device, gcst);
+	RELOC_PTR(gs_state, dfilter_stack);
     }
 }
 RELOC_PTRS_END
@@ -279,6 +278,7 @@ gs_state_alloc(gs_memory_t * mem)
     pgs->in_charpath = (gs_char_path_mode) 0;
     pgs->show_gstate = 0;
     pgs->level = 0;
+    pgs->dfilter_stack = 0;
     pgs->transparency_group_stack = 0;
     if (gs_initgraphics(pgs) >= 0)
 	return pgs;
