@@ -1,23 +1,20 @@
 /* Copyright (C) 1998, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gstext.c,v $ $Revision: 1.3.2.3 $ */
+/*$Id: gstext.c,v 1.6.2.2 2002/04/02 13:42:10 mpsuzuki Exp $ */
 /* Driver text interface support */
 #include "memory_.h"
 #include "gstypes.h"
@@ -82,15 +79,16 @@ RELOC_PTRS_END
 
 private ENUM_PTRS_WITH(text_enum_enum_ptrs, gs_text_enum_t *eptr)
 {
-    index -= 7;
+    index -= 8;
     if (index <= eptr->fstack.depth)
 	ENUM_RETURN(eptr->fstack.items[index].font);
     index -= eptr->fstack.depth + 1;
      return ENUM_USING(st_gs_text_params, &eptr->text, sizeof(eptr->text), index);
 }
 case 0: return ENUM_OBJ(gx_device_enum_ptr(eptr->dev));
-ENUM_PTR3(1, gs_text_enum_t, pis, orig_font, path);
-ENUM_PTR3(4, gs_text_enum_t, pdcolor, pcpath, current_font);
+case 1: return ENUM_OBJ(gx_device_enum_ptr(eptr->imaging_dev));
+ENUM_PTR3(2, gs_text_enum_t, pis, orig_font, path);
+ENUM_PTR3(5, gs_text_enum_t, pdcolor, pcpath, current_font);
 ENUM_PTRS_END
 
 private RELOC_PTRS_WITH(text_enum_reloc_ptrs, gs_text_enum_t *eptr)
@@ -99,6 +97,7 @@ private RELOC_PTRS_WITH(text_enum_reloc_ptrs, gs_text_enum_t *eptr)
 
     RELOC_USING(st_gs_text_params, &eptr->text, sizeof(eptr->text));
     eptr->dev = gx_device_reloc_ptr(eptr->dev, gcst);
+    eptr->imaging_dev = gx_device_reloc_ptr(eptr->imaging_dev, gcst);
     RELOC_PTR3(gs_text_enum_t, pis, orig_font, path);
     RELOC_PTR3(gs_text_enum_t, pdcolor, pcpath, current_font);
     for (i = 0; i <= eptr->fstack.depth; i++)
@@ -154,6 +153,7 @@ gs_text_enum_init(gs_text_enum_t *pte, const gs_text_enum_procs_t *procs,
 
     pte->text = *text;
     pte->dev = dev;
+    pte->imaging_dev = NULL;
     pte->pis = pis;
     pte->orig_font = font;
     pte->path = path;
@@ -545,6 +545,7 @@ void
 gx_default_text_release(gs_text_enum_t *pte, client_name_t cname)
 {
     rc_decrement_only(pte->dev, cname);
+    rc_decrement_only(pte->imaging_dev, cname);
 }
 void
 rc_free_text_enum(gs_memory_t * mem, void *obj, client_name_t cname)
