@@ -1,23 +1,20 @@
 /* Copyright (C) 1996, 2000 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gstype2.c,v $ $Revision: 1.7.2.2 $ */
+/*$Id: gstype2.c,v 1.10.2.3 2002/04/18 21:10:48 giles Exp $ */
 /* Adobe Type 2 charstring interpreter */
 #include "math_.h"
 #include "memory_.h"
@@ -56,7 +53,7 @@
       }\
   END
 private int
-type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state * ipsp,
+type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state_t * ipsp,
 	  bool explicit_width)
 {
     fixed wx;
@@ -69,12 +66,8 @@ type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state * ipsp,
 	wx = pcis->pfont->data.defaultWidthX;
     gs_type1_sbw(pcis, fixed_0, fixed_0, wx, fixed_0);
     /* Back up the interpretation pointer. */
-    {
-	ip_state *ipsp = &pcis->ipstack[pcis->ips_count - 1];
-
-	ipsp->ip--;
-	decrypt_skip_previous(*ipsp->ip, ipsp->dstate);
-    }
+    ipsp->ip--;
+    decrypt_skip_previous(*ipsp->ip, ipsp->dstate);
     /* Save the interpreter state. */
     pcis->os_count = csp + 1 - cstack;
     pcis->ips_count = ipsp - &pcis->ipstack[0] + 1;
@@ -132,7 +125,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
     fixed cstack[ostack_size];
     cs_ptr csp;
 #define clear CLEAR_CSTACK(cstack, csp)
-    ip_state *ipsp = &pcis->ipstack[pcis->ips_count - 1];
+    ip_state_t *ipsp = &pcis->ipstack[pcis->ips_count - 1];
     register const byte *cip;
     register crypt_state state;
     register int c;
@@ -166,6 +159,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
     if (str == 0)
 	goto cont;
     ipsp->char_string = *str;
+    ipsp->free_char_string = 0;	/* don't free caller-supplied strings */
     cip = str->data;
   call:state = crypt_charstring_seed;
     if (encrypted) {
@@ -225,16 +219,31 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		return_error(gs_error_invalidfont);
 	    case c_callsubr:
 		c = fixed2int_var(*csp) + pdata->subroutineNumberBias;
-		code = (*pdata->procs.subr_data)
+		code = pdata->procs.subr_data
 		    (pfont, c, false, &ipsp[1].char_string);
-	      subr:if (code < 0)
-		    return_error(code);
+	      subr:if (code < 0) {
+	            /* Calling a Subr with an out-of-range index is clearly a error:
+	             * the Adobe documentation says the results of doing this are
+	             * undefined. However, we have seen a PDF file produced by Adobe
+	             * PDF Library 4.16 that included a Type 2 font that called an
+	             * out-of-range Subr, and Acrobat Reader did not signal an error.
+	             * Therefore, we ignore such calls.
+	             */
+                    cip++;
+                    goto top;
+                }
 		--csp;
 		ipsp->ip = cip, ipsp->dstate = state;
 		++ipsp;
+		ipsp->free_char_string = code;
 		cip = ipsp->char_string.data;
 		goto call;
 	    case c_return:
+		if (ipsp->free_char_string > 0)
+		    gs_free_const_string(pfont->memory,
+					 ipsp->char_string.data,
+					 ipsp->char_string.size,
+					 "gs_type2_interpret");
 		--ipsp;
 		goto cont;
 	    case c_undoc15:
@@ -470,7 +479,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		    ipsp->ip = cip;
 		    ipsp->dstate = state;
 		    if (c == c2_cntrmask) {
-/****** NYI ******/
+			/****** NYI ******/
 		    } else {	/* hintmask or equivalent */
 			if_debug0('1', "[1]hstem hints:\n");
 			enable_hints(&pcis->hstem_hints, mask);
@@ -547,7 +556,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_const_string * str,
 		goto pushed;
 	    case c2_callgsubr:
 		c = fixed2int_var(*csp) + pdata->gsubrNumberBias;
-		code = (*pdata->procs.subr_data)
+		code = pdata->procs.subr_data
 		    (pfont, c, true, &ipsp[1].char_string);
 		goto subr;
 	    case cx_escape:
@@ -765,7 +774,8 @@ flex:			{
 			cnext;
 		    case ce2_hflex1:
 			csp[4] = fixed_half;	/* fd/100 */
-			csp[2] = *csp, csp[3] = 0;	/* dx6, dy6 */
+			csp[2] = *csp;          /* dx6 */
+			csp[3] = -(csp[-7] + csp[-5] + csp[-1]);	/* dy6 */
 			*csp = csp[-2], csp[1] = csp[-1];	/* dx5, dy5 */
 			csp[-2] = csp[-3], csp[-1] = 0;		/* dx4, dy4 */
 			csp[-3] = 0;	/* dy3 */
