@@ -1,25 +1,23 @@
 /* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gxchar.c,v $ $Revision: 1.2.2.3 $ */
+/*$Id: gxchar.c,v 1.5.2.5 2002/04/02 13:52:29 mpsuzuki Exp $ */
 /* Default implementation of text writing */
 #include "gx.h"
+#include "errors.h"
 #include "memory_.h"
 #include "string_.h"
 #include "gserrors.h"
@@ -40,14 +38,14 @@
 
 /* Define whether or not to cache characters rotated by angles other than */
 /* multiples of 90 degrees. */
-private bool CACHE_ROTATED_CHARS = true;
+private const bool CACHE_ROTATED_CHARS = true;
 
 /* Define whether or not to oversample characters at small sizes. */
-private bool OVERSAMPLE = true;
+private const bool OVERSAMPLE = true;
 
 /* Define the maximum size of a full temporary bitmap when rasterizing, */
 /* in bits (not bytes). */
-private uint MAX_TEMP_BITMAP_BITS = 80000;
+private const uint MAX_TEMP_BITMAP_BITS = 80000;
 
 /* Define whether the show operation uses the character outline data, */
 /* as opposed to just needing the width (or nothing). */
@@ -60,7 +58,7 @@ extern_st(st_gs_text_enum);
 extern_st(st_gs_state);		/* only for testing */
 private 
 ENUM_PTRS_BEGIN(show_enum_enum_ptrs)
-     return ENUM_USING(st_gs_text_enum, vptr, size, index);
+     return ENUM_USING(st_gs_text_enum, vptr, size, index - 5);
 ENUM_PTR(0, gs_show_enum, pgs);
 ENUM_PTR(1, gs_show_enum, show_gstate);
 ENUM_PTR3(2, gs_show_enum, dev_cache, dev_cache2, dev_null);
@@ -243,15 +241,20 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 
 	if (code < 0)
 	    return code;
+	if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE)) /* cshow */
+            return code;
 	return set_cache_device(penum, pgs, pw[2], pw[3], pw[4], pw[5]);
     }
     case TEXT_SET_CACHE_DEVICE2: {
 	int code;
+	bool retry = (penum->width_status == sws_retry);
 
 	if (gs_rootfont(pgs)->WMode) {
 	    float vx = pw[8], vy = pw[9];
 	    gs_fixed_point pvxy, dvxy;
-	    cached_char *cc;
+
+	    gs_fixed_point rewind_pvxy;
+	    int rewind_code;
 
 	    if ((code = gs_point_transform2fixed(&pgs->ctm, -vx, -vy, &pvxy)) < 0 ||
 		(code = gs_distance_transform2fixed(&pgs->ctm, vx, vy, &dvxy)) < 0
@@ -259,19 +262,32 @@ gx_show_text_set_cache(gs_text_enum_t *pte, const double *pw,
 		return 0;		/* don't cache */
 	    if ((code = set_char_width(penum, pgs, pw[6], pw[7])) < 0)
 		return code;
+	    if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE))
+		return code;
 	    /* Adjust the origin by (vx, vy). */
 	    gx_translate_to_fixed(pgs, pvxy.x, pvxy.y);
 	    code = set_cache_device(penum, pgs, pw[2], pw[3], pw[4], pw[5]);
-	    if (code != 1)
+	    if (code != 1) {
+	        if (retry) {
+		   rewind_code = gs_point_transform2fixed(&pgs->ctm, vx, vy, &rewind_pvxy);
+		   if (rewind_code < 0) {
+		       /* If the control passes here, something is wrong. */
+		       return_error(e_unregistered);
+		   }
+		   /* Rewind the origin by (-vx, -vy) if the cache is failed. */
+		   gx_translate_to_fixed(pgs, rewind_pvxy.x, rewind_pvxy.y);
+		}
 		return code;
+	    }
 	    /* Adjust the character origin too. */
-	    cc = penum->cc;
-	    cc->offset.x += dvxy.x;
-	    cc->offset.y += dvxy.y;
+	    (penum->cc)->offset.x += dvxy.x;
+	    (penum->cc)->offset.y += dvxy.y;
 	} else {
 	    code = set_char_width(penum, pgs, pw[0], pw[1]);
 	    if (code < 0)
 		return code;
+	    if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE))
+		return code;
 	    code = set_cache_device(penum, pgs, pw[2], pw[3], pw[4], pw[5]);
 	}
 	return code;
@@ -289,7 +305,7 @@ set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
 {
     int code;
 
-    if (penum->width_status != sws_none)
+    if (penum->width_status != sws_none && penum->width_status != sws_retry)
 	return_error(gs_error_undefined);
     if ((code = gs_distance_transform2fixed(&pgs->ctm, wx, wy, &penum->wxy)) < 0)
 	return code;
@@ -301,6 +317,8 @@ set_char_width(gs_show_enum *penum, gs_state *pgs, floatp wx, floatp wy)
     } else {
 	penum->width_status = sws_no_cache;
     }
+    if (SHOW_IS_ALL_OF(penum, TEXT_DO_NONE | TEXT_INTERVENE)) /* cshow */
+	gs_nulldevice(pgs);
     return !SHOW_IS_DRAWING(penum);
 }
 
@@ -600,6 +618,7 @@ show_update(gs_show_enum * penum)
     /* Update position for last character */
     switch (penum->width_status) {
 	case sws_none:
+        case sws_retry:	  
 	    /* Adobe interpreters assume a character width of 0, */
 	    /* even though the documentation says this is an error.... */
 	    penum->wxy.x = penum->wxy.y = 0;
@@ -1043,7 +1062,7 @@ gx_show_text_retry(gs_text_enum_t *pte)
 	penum->cc = 0;
     }
     gs_grestore(penum->pgs);
-    penum->width_status = sws_none;
+    penum->width_status = sws_retry;
     penum->log2_scale.x = penum->log2_scale.y = 0;
     return 0;
 }
