@@ -1,23 +1,20 @@
 /* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
+  This software is provided AS-IS with no warranty, either express or
+  implied.
   
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
+  This software is distributed under license and may not be copied,
+  modified or distributed except as expressly authorized under the terms
+  of the license contained in the file LICENSE in this distribution.
   
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  For more information about licensing, please refer to
+  http://www.ghostscript.com/licensing/. For information on
+  commercial licensing, go to http://www.artifex.com/licensing/ or
+  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
+  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$RCSfile: gxifast.c,v $ $Revision: 1.2.2.3 $ */
+/*$Id: gxifast.c,v 1.4.2.1 2002/02/22 19:45:58 ray Exp $ */
 /* Fast monochrome image rendering */
 #include "gx.h"
 #include "memory_.h"
@@ -146,17 +143,19 @@ gs_image_class_1_simple(gx_image_enum * penum)
     penum->unpack = sample_unpack_copy;
     penum->unpack_bps = 8;
     if (penum->use_mask_color) {
-	/* Adjust the colors and mask status accordinate to the mask color. */
+	/*
+	 * Set the masked color as 'no_color' to make it transparent
+	 *  according to the mask color range and the decoding.
+	 */
 	penum->masked = true;
 	if (penum->mask_color.values[0] == 1) {
-	    /* v0 = v1 = 1, 1 is transparent. */
-	    memcpy(&penum->map[0].table.lookup4x1to32[0],
-		   lookup4x1to32_inverted, 16 * 4);
-	    penum->icolor1 = penum->icolor0;
+	    /* if v0 == 1, 1 is transparent since v1 must be == 1 to be a valid range */
+	    color_set_pure(penum->map[0].inverted ? &penum->icolor0 : &penum->icolor1,
+			gx_no_color_index);
 	} else if (penum->mask_color.values[1] == 0) {
-	    /* v0 = v1 = 0, 0 is transparent. */
-	    memcpy(&penum->map[0].table.lookup4x1to32[0],
-		   lookup4x1to32_identity, 16 * 4);
+	    /* if v1 == 0, 0 is transparent since v0 must be == 0 to be a valid range */
+	    color_set_pure(penum->map[0].inverted ? &penum->icolor1 : &penum->icolor0,
+			gx_no_color_index);
 	} else {
 	    /*
 	     * The only other possible in-range value is v0 = 0, v1 = 1.
@@ -164,7 +163,6 @@ gs_image_class_1_simple(gx_image_enum * penum)
 	     */
 	    rproc = image_render_skip;
 	}
-	color_set_pure(&penum->icolor0, gx_no_color_index);
 	penum->map[0].decoding = sd_none;
     }
     return rproc;
@@ -327,7 +325,7 @@ image_simple_expand(byte * line, int line_x, uint raster,
     dxx4 = xl.step;
     dda_step_add(dxx4, xl.step);
     /* egcc - 2.91.66 generates incorrect code for
-     * dda_step_add(dxx4, dxx4);
+     * dda_step_add(dxx4, dxx4); 
      * Using the temp variable.
      */
     dxx8 = dxx4;
@@ -659,16 +657,6 @@ image_render_landscape(gx_image_enum * penum, const byte * buffer, int data_x,
     byte *row;
     const byte *orig_row = 0;
     bool y_neg = penum->dxy < 0;
-    gx_device_color * const pdc0 = &penum->icolor0;
-    gx_device_color * const pdc1 = &penum->icolor1;
-    int code;
-
-    if ((!DC_IS_NULL(pdc0) &&
-	 (code = gx_color_load(pdc0, penum->pis, dev)) < 0) ||
-	(!DC_IS_NULL(pdc1) &&
-	 (code = gx_color_load(pdc1, penum->pis, dev)) < 0)
-	)
-	return code;
 
     if (is_fneg(penum->matrix.yx))
 	ix += iw, iw = -iw, xinc = -1;
